unit Annealer;
{$mode objfpc}{$H+}
interface // this section makes stuff visible to other units
uses Classes, SysUtils, StdCtrls, StrUtils, SchBldrUtils, Forms,Builder;
// Forms provides access to Application.ProcessMessages

procedure SaveAnnealPageSetUp(Memo: TMemo);
procedure AnalyzeSchematic(var CfgMemo,NetMemo,AnlMemo: TMemo);
procedure OptimizeSchematic(var AnlMemo: TMemo);

implementation // anything appearing only here is invisible to other units
uses SchBldrMain; // if needed must go here to prevent circular reference

var // these variables are global only to the Analyzer unit
  SchStart,Xpos,Ypos,Len,Nets,Links,Symbols,H,V: integer; NetType: string;
  SymNetList,TxtNetList,IDpinsNets,IDnetsList,NetIDsList,IDidsList,
  SchField,OptiList: TStringList;
  NowOptimizing,QuitOptimizing: boolean;

////////////////////////////////////////////////////////////////////////////////

// these are used for capturing various Anneal Memo header lines
var Hdr1,Hdr2: string; Ftr1: array [0..11]of string;
procedure SaveAnnealPageSetUp(Memo: TMemo); var i: integer;
begin with Memo do begin // save the Anneal page configuration
  Hdr1:= Lines[0]; Hdr2:= Lines[1]; for i:=5 to 14 do Ftr1[i-5]:= Lines[i];
  OptiList:= TStringList.Create; // create global string lists
end;end;

// split a netlist into trimmed lists of only symbol items and only text items
procedure SplitNetList(NetMemo: TMemo); // fills globals SymNetList TxtNetList
  var i: integer; nStr: string;
begin with NetMemo do begin
  SymNetList.Clear; TxtNetList.Clear; // make sure the target lists start empty
  for i:= 1 to Lines.Count-1 do begin // line[0] is always a comment?
    nStr:= Trim(Lines[i]); // copy and trim the netlist line
    if(Length(nStr)>0)then if(nStr[1]in DeviceIDs) // split non empty lines...
      then SymNetList.Append(nStr)  // into a symbols only list...
      else TxtNetList.Append(nStr); // and a text only list
end;end;end;

// use config data to parse out list of ID Pins NetNames = IDpinsNets
procedure ParseLinkedNets(CfgMemo: TMemo); // global IDpinsNets
  var CfgStr,NetStr,ID2,Value,SrtStr: string;
    i,j,k,Pins: integer; NoSymbolFound: boolean;
begin with SymNetList do begin
  IDpinsNets.Clear; // make sure it starts empty
  for i:=0 to Count-1 do begin// checking the symbol-only netlist
    NetStr:= Strings[i]; j:=3; // reset the symbol index
    k:= Pos('(',NetStr); if k>0 then System.Insert(' ',NetStr,k); // fix "poly()"
    repeat inc(j); NoSymbolFound:= (j>=CfgMemo.Lines.Count);
      CfgStr:= CfgMemo.Lines[j]; // go get the config ID-indexed data
      if(length(CfgStr)=0)or not(CfgStr[1]in DeviceIDs) then continue;
      // the configuration line is empty or is not a symbol
      if(CfgStr[1]=UpCase(NetStr[1]))then begin // found an ID match, so ...
        ID2:= CpyWrd(2,CfgStr); // get the 2nd ID,
        Pins:= nt(CpyWrd(6,CfgStr)); // the number of pins (connections)
        Value:= CpyWrd(2+Pins,NetStr); // and the component value
        if (Value='')and(CfgStr[1]in['M','Q']) then begin // whoops, it's
          dec(Pins); Value:= CpyWrd(2+Pins,NetStr); // not a 4 pin transistor
        end; // BJTs and MOSFETs may be only 3 pins (this needs work?)
        if (ID2='.')or(Pos(ID2,LowerCase(Value))=1) then begin // (ID2 match)
          SrtStr:= CpyWrd(1,NetStr); // first get the ID
          SrtStr:= SrtStr+' '+st(Pins); // then append number of links
          // now, extract the netnames connected to pins (the links)
          for k:= 1 to Pins do SrtStr:= SrtStr+CpyWrd(1+k,NetStr)+' ';
          IDpinsNets.Append(SrtStr); // save it to the reworked sort-list
          break; // ID2 match means found the symbol, so quit looking further
      end;end;
    until NoSymbolFound; // skipped over by break once a symbol is found
    if NoSymbolFound then beep; // error, this should never happen
end;end;end;

// IDpinsNets -> IDnetsList + NetIDsList
procedure ParsePairs(var Count: integer); // IDpinsNets IDnetsList NetIDsList
  var i,pin,Pins: integer; ID: string;
begin
// this first part breaks out the lists of ID/Net and Net/ID pairs
  IDnetsList.Clear; NetIDsList.Clear; // make sure the lists start empty
  with IDpinsNets do for i:= 0 to Count-1 do begin
    ID:= CpyWrd(1,Strings[i]); // get the ID from the sort netlist
    Pins:= nt(CpyWrd(2,Strings[i])); // and get the number of connections
    for pin:= 1 to Pins do begin // and create both expanded lists
      IDnetsList.Append(ID+' '+CpyWrd(2+pin,Strings[i]));
      NetIDsList.Append(CpyWrd(2+pin,Strings[i])+' '+ID);
  end;end; Count:= IDnetsList.Count; // get the count for the pairs
  IDnetsList.Sort; NetIDsList.Sort;  // then sort each list
// this next part consolodates the lists by first word (ID or Net)
  ConsolodatePairs(IDnetsList); ConsolodatePairs(NetIDsList);
end;

// inserts into NetIDsList the missing fields, NetType and NumIDs
// NetIDsList: NetType NetName NumIDs IDsList
procedure InsertNetTypeAndNumIDs(CfgMemo: TMemo);
  var i,j,NumIDs,n: integer; Net,NIDstr,CfgStr: string; NotFound: boolean;
begin for i:= 0 to NetIDsList.Count-1 do begin
  NIDstr:= NetIDsList[i]; // 'Insert' requires lines be converted to strings
  NumIDs:= WordCount(NIDstr,WhtSp)-1; // get the number (+1) of IDs
  n:= WordPosition(2,NIDstr,WhtSp); // figure out where to stuff the data
  Insert(#9#9+st(NumIDs)+#9,NIDstr,n); // insert the number of links (with tabs)
  // this next part checks CfgMemo for any special NetType name matches
  Net:= CpyWrd(1,NIDstr); NetType:= ' Net';
  j:= 0; repeat inc(j); NotFound:= (j>=CfgMemo.Lines.Count);
    CfgStr:= CfgMemo.Lines[j]; // go get the config ID-indexed data
    if(length(CfgStr)=0)or(CfgStr[1]<>'|') then continue; // because...
    // ...the configuration line is empty or is not an nType key
    if IsWildNumMatch(Net,CpyWrd(2,CfgStr))then begin // found a match
      NetType:= ' '+CpyWrd(3,CfgStr);
      break; end; // found the symbol, so quit looking further
  until NotFound; // skipped over by break once a symbol is found
  //if NotFound then beep; // error, this should never happen
  NIDstr:= NetType+' '+#9+NIDstr; // the net_type+tab goes at the front
  NetIDsList[i]:= NIDstr; // put the cooked string back in the line
end;end; // later before displaying, sort by number of links (fewest first)

// IDnetsList + NetIDsList -> IDidsList  (blocks all GND links)
// IDnetsList: ID NetsList,  NetIDsList: NetType NetName NumIDs IDsList
procedure ParseLinks; // IDpinsNets IDnetsList NetIDsList
  var i,j,IDcnt,ids,d,NetCnt,nets,n: integer; ID,IDd,Net,NetJ: string;
begin // fold the two lists together to create a list of IDs by ID
  IDcnt:= IDnetsList.Count-1; NetCnt:= NetIDsList.Count-1;
  for i:= 0 to IDcnt do begin
    ID:= CpyWrd(1,IDnetsList[i]); // get the self-ID for the line
    nets:= WordCount(IDnetsList[i],WhtSp); // count the nets+1 for this ID
    j:= 0; for n:= 2 to 2{nets} do begin // the word position of each net in turn
      Net:= CpyWrd(n,IDnetsList[i]); // read the net name from the list
      while(j<NetCnt)do begin
        NetJ:= CpyWrd(2,NetIDsList[j]); // NetNames are 2nd word
        if(NetJ=Net)then begin // we found a matching net!
          NetType:= LowerCase(CpyWrd(1,NetIDsList[j])); // NetTypes are 1st word
          case NetType of
            'port': ; // ignore ground and other port type nets
            'top' : IDidsList.Append(ID+' '+'zt');
            'bot' : IDidsList.Append(ID+' '+'zb');
            'left': IDidsList.Append(ID+' '+'zl');
            'rht' : IDidsList.Append(ID+' '+'zr');
          end;
          if (Net<>'0')and(NetType='net')then begin
            ids:= CpyInt(3,NetIDsList[j]); // count the IDs
            // alternate to above = WordCount(NetIDsList[j],WhtSp);
            for d:= 1 to ids do begin // the word position of each ID in turn
              IDd:= CpyWrd(d+3,NetIDsList[j]); if not(IDd=ID) // ignore self ID
                then IDidsList.Append(ID+' '+IDd); end;
          end; break; end;
        inc(j);
    end;end;
  end; IDidsList.Sort; ConsolodatePairs(IDidsList);
end;

// Now pick IDs from SymNetList to fill in the SchGrid and OptiList, first top
// to bottom, then left to right, populating the final positions with dot IDs.
// Data structure formats for reference:
// NetMemo -> Netlist -> TxtNetList + SymNetList
// SymNetList + H+V -> SchGrid + OptiList: ID x y (Len NumIDs IDsList)
procedure BuildGridSch(var SchGrid: TStringList; H,V: integer);
  var Hdr,Sym: string; s,x,y: integer;
begin with SchGrid do begin // before starting, clear and report field size
  {Clear;} Append('Schematic Symbol Field: '+st(H)+'x '+st(V));
  // then create the correct width header/footer boarder
  Hdr:=' +'#9; for x:=1 to H do Hdr:= Hdr+'- -'#9; Hdr:= Hdr+'+';
  Append(Hdr); // write top boarder of field
  for y:= 1 to V do Append('  :'#9); // write all the left boarders of field
  Symbols:= SymNetList.Count;  s:= 0; // get count and zero symbol index
  for x:= 1 to H do for y:= 1 to V do begin // process each column in turn
    // get the IDs from SymNetList until out then fill with dot/nul symbol
    if (s)<Symbols then Sym:= CpyWrd(1,SymNetList[s]) else Sym:= '.';
    OptiList.Append(Sym+' '+st(x)+st(y)); // initialize list with ID x y
    SchGrid[1+y]:= SchGrid[1+y]+Sym+#9; // add symbols to left boarder of field
    inc(s); // move to next symbol
  end;
  for y:= 1 to V do SchGrid[1+y]:= SchGrid[1+y]+':'; // add on right boarder
  Append(Hdr); // write bottom boarder of field
end;end;

// works on global OptiList which must start out prefilled with ID x y.
// Data structure formats for reference:
// NetMemo -> Netlist -> TxtNetList + SymNetList
// SymNetList + H+V -> SchGrid + OptiList: ID x y (Len NumIDs IDsList)
// SymNetList + CfgMemo -> IDpinsNets: ID Pins NetNames
// IDpinsNets + CfgMemo -> NetIDsList: NetType NetName NumIDs IDsList
// OptiList: ID x y Len NumIDs IDsList
procedure OptiListSetUp; var x,i: integer; IDline: string;
begin x:= 0; while CpyWrd(1,OptiList[x])='.' do begin
    OptiList[x]:= OptiList[x]+' 0 0'; inc(x); end;
  for i:= 0 to IDidsList.Count-1 do begin
    IDline:= IDidsList[i]; ExtWrd(1,IDLine); // remove the redundant ID
    // IDs should all match now, but it might be safer to check at each step
    OptiList[i+x]:= OptiList[i+x]+'0 '+st(WordCount(IDLine,WhtSp))+IDLine;
  end;
  OptiList.Append('zb '+st(H div 2)+st(V+1)+'0 0');
  OptiList.Append('zl '+st(0)+st(V div 2)+'0 0');
  OptiList.Append('zr '+st(H+1)+st(V div 2)+'0 0');
  OptiList.Append('zt '+st(H div 2)+st(0)+'0 0');
end;

// OptiList: ID x y Len NumIDs IDsList
function NetLen(n: integer): integer; // returns net length of line(n)
var i,d,x,y: integer; ID,IDx: string;
begin with OptiList do begin NetLen:= 0; i:= 0;
  x:= nt(CpyWrd(2,Strings[n])); // get the Xpos from the list
  y:= nt(CpyWrd(3,Strings[n])); // get the Ypos from the list
  for d:= 1 to nt(CpyWrd(5,Strings[n])) do begin
    IDx:= CpyWrd(5+d,Strings[n]); // get the next linked ID from the list
    repeat inc(i); ID:= CpyWrd(1,Strings[i]);
    until (ID=IDx)or(i>=Count);
    inc(NetLen,abs(x-nt(CpyWrd(2,Strings[i])))+abs(y-nt(CpyWrd(3,Strings[i]))));
    dec(i);
end;end;end;

// calculate OptiList individual net lengths, ignoring nul postions
procedure CalcNetLengths; var i: integer; line: string;
begin with OptiList do begin
  for i:= 0 to Count-1 do if(CpyWrd(1,Strings[i])<>'.')then begin
    line:= Strings[i]; RplInt(NetLen(i),4,line); Strings[i]:= line; end;
end;end;

// tallies total net length and writes it to AnlMemo
procedure UpdateTotalNetLen(var Memo: Tmemo); // global OptiList
  var i,Len: integer;
  begin with OptiList do begin
    Len:= 0; // update total net length counter
    for i:= 0 to Count-1 do inc(Len,nt(CpyWrd(4,Strings[i])));
    Memo.Append('Total net length: '+st(Len));
  end;end;

// works on global OptiList which must start out prefilled with ID x y.
// Data structure formats for reference:
// NetMemo -> Netlist -> TxtNetList + SymNetList
// SymNetList + H+V -> SchGrid + OptiList: ID x y (Len NumIDs IDsList)
// SymNetList + CfgMemo -> IDpinsNets: ID Pins NetNames
// IDpinsNets + CfgMemo -> NetIDsList: NetType NetName NumIDs IDsList
// OptiList: ID x y Len NumIDs IDsList
procedure WriteNetList(var NL: TStringList); // global OptiList, V
  var i,j,n: integer; const x=2; y=3;
begin
  NL.Clear; for i:= 1 to OptiList.Count do NL.Append('~');
  n:= 0; while CpyWrd(1,OptiList[n])='.' do inc(n);
  SymNetList.Sort; for i:= 0 to SymNetList.Count-1 do begin
    j:= (CpyInt(x,OptiList[i+n])-1)*V+(CpyInt(y,OptiList[i+n])); // vert xy index
    NL[j]:= SymNetList[i]; // reorder the netlist per the optimization sort
  end;
  NL.AddStrings(TxtNetList); // add back all the commands, comments and text
end;

// this is a custom sort function for sorting OptiList by x y values
function XYsrt(TSL: TStringList; a,b: integer): integer; // global OptiList H V
  var Xa,Ya,La,Xb,Yb,Lb: integer;
  const WS = [#0..' ']; x = 2; y = 3; // x y fields
  begin
    Val(ExtractWord(x,TSL[a],WS),Xa); Val(ExtractWord(y,TSL[a],WS),Ya);
    Val(ExtractWord(x,TSL[b],WS),Xb); Val(ExtractWord(y,TSL[b],WS),Yb);
    La:= H*Ya+Xa; Lb:= H*Yb+Xb;
    if La<Lb then XYsrt:=-1 else if La=Lb then XYsrt:= 0 else XYsrt:=1;
  end;

// this is a custom sort function for sorting NetIDsList by NumIDs
function LnkSrt(TSL: TStringList; a,b: integer): integer;
  var Na,Nb: integer; const WS = [#0..' ']; n=3; // n is NumIDs field#
  begin Val(ExtractWord(n,TSL[a],WS),Na); Val(ExtractWord(n,TSL[b],WS),Nb);
    if Na<Nb then LnkSrt:=-1 else if Na=Nb then LnkSrt:= 0 else LnkSrt:=1;
  end;

////////////////////////////////////////////////////////////////////////////////
procedure AnalyzeSchematic(var CfgMemo,NetMemo,AnlMemo: TMemo);
var Totals: string; i: integer;
begin
  if NowOptimizing then begin QuitOptimizing:= true; end;
//    i:= 0; repeat Application.ProcessMessages; until i>1000 end;
  SymNetList:= TStringList.Create; TxtNetList:= TStringList.Create;
  IDpinsNets:= TStringList.Create; IDnetsList:= TStringList.Create;
  NetIDsList:= TStringList.Create; IDidsList:= TStringList.Create;
  SchField:= TStringList.Create; OptiList:= TStringList.Create;
try // ensures eventual clean up of the temporary stringlists made above
  // first split out a copy of the user netlist w/o comments (symbol lines only)
  SplitNetList(NetMemo); // NetMemo (raw netlist) -> SymNetList & TxtNetList
  // then parse it down to just the minimum necessary netlist data
  ParseLinkedNets(CfgMemo); // -> IDpinsNets: ID Pins LinkedNets
  // next parse the data into the sorted master lists
  ParsePairs(Links); // -> IDnetsList + NetIDsList -> IDidsList (w/o GND links)
  // AnlMemo.Append(IDnetsList.Text); AnlMemo.Append(NetIDsList.Text); // temp

  // now fill in NetType and NumIDs count
  InsertNetTypeAndNumIDs(CfgMemo); // -> NetIDsList: NetType NetName NumIDs IDsList
  // generate the IDidsList fully cooked by NetType
  ParseLinks; // -> IDnetsList + NetIDsList -> IDidsList (per NetTypes)
  //AnlMemo.Append(IDidsList.Text); // temp

  // next reverse sort the list by the number of Links (field 3)
  NetIDsList.CustomSort(@LnkSrt); // -> NetIDsList sorted by num links
  // now generate the various count totals for the netlist analysis
  Nets:= NetIDsList.Count; {Links from ParseLinks} Symbols:= SymNetList.Count;
  // Assemble Totals: tNets tLength tLinks tSymbols
  Totals:= 'Total: '#9+st(Nets)+#9#9+st(Links)+#9+st(Symbols);
  // find the grid schematic dimensions H x V (field to be H+1 x V+1)
  CalcHV(H,V,Symbols); // calculate H V based on symbol count
  // build SchField list (contains the "schematic" lines to be optimized)
  // add in x y and length fields
  BuildGridSch(SchField,H,V); // build initial SchField and OptiList
  OptiList.Sort; // the raw netlist was unsorted at readin
  OptiListSetUp; // get all the missing fields
  OptiList.Sort; // the raw netlist was unsorted at readin
  CalcNetLengths; // fill in the lengths now that we have x y info
//  AnlMemo.Append(OptiList.Text); // temp

  ////////////////////////////////////////////////////////////////////////////
  // Now Write To The Display (Anneal Memo)
  // clear the display and write the accumulated statistics
  with AnlMemo do with Lines do begin Clear;
    Append(Hdr1); Append(Hdr2); // write header and separator
    Append(NetIDsList.Text); Delete(Count-1); // kill unwanted ending CR
    Append(Hdr2); Append(Totals); // write closing separator and totals
    for i:= 0 to 11 do Append(Ftr1[i]); // write the footer note and legend
    // Keep a global offset to the line on AnlMemo where it starts.
    SchStart:= Count; // remember where the symbol field should start
    Append(SchField.Text); // this inserts an ending CR... which is wanted
    // Append('Iterations without improvement: 0'); Append('');
    UpdateTotalNetLen(AnlMemo); // tallies total and writes it to AnlMemo
    // Append(OptiList.Text); // temporary to get a look at the data
  end;
  // the user may now edit NType and perhaps nName (generates a S&R list?)
finally // release all the memory used by the local stringlists...
  {SymNetList.free; TxtNetList.free;} IDpinsNets.free; IDnetsList.free;
  {NetIDsList.free;} IDidsList.free; SchField.free; {OptiList.free;}
end;end; // but don't release the global lists needed by the optimizer

// OptiList: ID x y Len NumIDs IDsList
function LenXY(x,y,n: integer): integer;
var i,id: integer; IDx,ss: string;
begin with OptiList do begin
  i:= 0; LenXY:= 0;
  for id:= 1 to nt(CpyWrd(5,Strings[n])) do begin
    IDx:= CpyWrd(5+id,Strings[n]);
    ss:= CpyWrd(1,Strings[i]);
    while(ss<>IDx)do begin inc(i);
      if i>=Count then break;
      ss:= CpyWrd(1,Strings[i]); end;
    inc(LenXY,abs(x-nt(CpyWrd(2,Strings[i])))+abs(y-nt(CpyWrd(3,Strings[i]))));
end;end;end;
// This searching algorithm depends on both lists being presorted.  When
// this is the case all IDs can be found in one pass instead of requiring
// a separate pass for each ID.  This is the advantage of presorted lists.
// Note: to designate an empty line in the schematic field, the empty line
// ID should be tilde '~' if sorting up, else if sorting down, use zero '0'.
// This ensures that the empty lines are never reached during the search.

// NetMemo -> Netlist -> TxtNetList + SymNetList
// SymNetList + CfgMemo -> IDpinsNets: ID Pins NetNames
// IDpinsNets + CfgMemo -> NetIDsList: NetType NetName NumIDs IDsList
// SchField
// CalcLenTSL -> OptiList
// OptiList: ID x y Len NumIDs IDsList (if x=0~ then dx=0, if y=0~ then dy=0)
const ID_=1; X_=2; Y_=3; L_=4; N_=5; BlockSch=true;
procedure OptimizeSchematic(var AnlMemo: TMemo);
var TempNetMemo: TMemo; TSL: TStringList; TS: TStrings;
var OL1,OL2,ID1,ID2,GS1,GS2: string; AnlMem: TStrings;
  n,m,HVm1,Idx1,Idx2,X1,Y1,X2,Y2,X3,Y3,L0,L1,L2,L3,dL,i,Len: integer;
  procedure AdjLinkedLen(OL: string); // uses global OptiList
    var i,id: integer; IDn: string;
    begin with OptiList do begin i:= 0;
      for id:= 1 to nt(CpyWrd(N_,OL)) do begin IDn:= CpyWrd(N_+id,OL);
        while(CpyWrd(ID_,Strings[i])<>IDn)do inc(i);
        X3:= nt(CpyWrd(X_,Strings[i])); Y3:= nt(CpyWrd(Y_,Strings[i]));
        dL:=abs(X1-X3)+abs(Y1-Y3)-abs(X2-X3)-abs(Y2-Y3);
        dec(i); RplInt(L2,L_,OL2); // stuff new Len
    end;end;end;
const ittr2=1000; ittr1=50*ittr2;
begin if NowOptimizing then QuitOptimizing:= true else with OptiList do begin
  m:= 0; NowOptimizing:= true; QuitOptimizing:= false;
  AnlMem:= AnlMemo.Lines; HVm1:= H*V-1; //(H+1)*(V+1)-1;
  n:= 0; repeat inc(n);
    Idx1:= random(HVm1); repeat Idx2:= random(HVm1); until Idx2<>Idx1;
    OL1:= Strings[Idx1]; OL2:= Strings[Idx2];
    X1:= nt(CpyWrd(X_,OL1)); X2:= nt(CpyWrd(X_,OL2));
    Y1:= nt(CpyWrd(Y_,OL1)); Y2:= nt(CpyWrd(Y_,OL2));
    L0:= nt(CpyWrd(L_,OL1))  +    nt(CpyWrd(L_,OL2));
    L1:= LenXY(X2,Y2,Idx1); L2:= LenXY(X1,Y1,Idx2);
    if((L1+L2)<L0)or(n=ittr2)
      then begin // swap data
        RplInt(X2,X_,OL1); RplInt(X1,X_,OL2); // swap X's
        RplInt(Y2,Y_,OL1); RplInt(Y1,Y_,OL2); // swap Y's
        RplInt(L1,L_,OL1); RplInt(L2,L_,OL2); // stuff in new lengths
        AdjLinkedLen(OL1); AdjLinkedLen(OL2); // adjust the linked ID lengths
        // swap the ID text in the grid schematic
        ID1:= CpyWrd(ID_,OL1); ID2:= CpyWrd(ID_,OL2);
        GS1:= AnlMem[SchStart+1+Y1]; RplWrd(ID2,1+X1,GS1);
        AnlMem[SchStart+1+Y1]:= GS1;
        GS2:= AnlMem[SchStart+1+Y2]; RplWrd(ID1,1+X2,GS2);
        AnlMem[SchStart+1+Y2]:= GS2;
        if(n<>ittr2)then n:= 0; // reset counter of itterations without improvement
        if m<10 then inc(m) else begin m:= 0; Application.ProcessMessages; end;
      end;
    Strings[Idx1]:= OL1; Strings[Idx2]:= OL2;
  until (QuitOptimizing)or(n>ittr1);
  if QuitOptimizing then QuitOptimizing:= false else begin
    //AnlMemo.Append(OptiList.Text); // temporary to get a look at the data
    UpdateTotalNetLen(AnlMemo); // tallies total and writes it to AnlMemo
    TempNetMemo:= TMemo.Create(Form1.AnlSht); TS:= TempNetMemo.Lines;
    TSL:= TStringList.Create;
    WriteNetList(TSL);
    for i:=0 to TSL.Count-1 do TS.Append(TSL[i]); // clumsy stuff into memo
    with Form1 do BuildSchematic(BlockSch,CfgMemo,TempNetMemo,BldMemo,SchMemo);
    //AnlMemo.Append(TSL.Text);
    TempNetMemo.Free; TSL.Free;
  end;
  NowOptimizing:= false;
end;end;


end. // anything beyond this point is ignored (notes and archive area)

Mirror and rotational translations
None    X  Y
Mirror -x +y
MV     +x -y
Flip   -x -y
??     +y +x
Right  -y +x
Left   +y -x
??     -y -x

//**************************************************************************
ANALYZER
Given a NetList (from NetMemo -> SymNetList)
Create a nTypeList (SymNetList+CfgMemo -> IDpinsNets -> AnlMemo)
Create the schematic symbol field (IDpinsNets -> SchField -> AnlMemo)
Check for any user input or changes

DATA
NetIDsList: nType nName nLen nLinks IDList
IDpinsNets: ID x y LinksLen LinksCount IDpinsNets
SchField: -> ID at any point

OPTIMIZER
Randomly perturb schematic (select two or three points to exchange)
Get the IDs from those two points (SchField)
Swap if allowed per NetIDsList
Calculate new length ID -> IDpinsNets -> Nets -> NetIDsList -> IDList

procedure OptiListSetUp; var IDL: TStringList;
  ci,nets,ni,si,IDs,id: integer; OptiLine,Net,NSline,Stats,ID0: string;
begin IDL:= TStringList.Create; try // ensures release of dynamic memory
  IDL.Sorted:= true; IDL.Duplicates:= dupIgnore; // add sorted only unique IDs
  IDL.StrictDelimiter:= true; IDL.Delimiter:=' ';
for ci:= 0 to OptiList.Count-1 do begin // ci = calc index
  OptiLine:= OptiList[ci]; // get the current line to be processed
  IDL.Clear; // start with a cleared IDList for each new line processed
  ID0:= CpyWrd(1,OptiLine); // get the self ID from OptiLine
  nets:= nt(CpyWrd(3,OptiLine)); // get the num of nets from OptiLine
  for ni:= 1 to nets do begin // ni = net index
    Net:= ExtWrd(4,OptiLine); // remove the next linked Net from OptiLine
    si:= NetIDsList.Count; // si = stat index
    repeat dec(si); NSline:= NetIDsList[si];
    until (CpyWrd(2,NSLine)=Net)or(si<=0); // check for net name match
    if(si=0)and(CpyWrd(2,NSLine)<>Net)then beep // beep if no match found
    else if(Net<>'0')and(Net<>'+10V')and(Net<>'-10V')then
    begin // now get IDs from NetIDsList and append them onto IDList
      Stats:= NetIDsList[si]; IDs:= nt(CpyWrd(2,Stats)); // get ID count
      for id:=1 to IDs do IDL.Add(CpyWrd(2+id,Stats)); // append IDs
  end;end;
  id:= IDL.Count; // id = ID index
  repeat dec(id); until (id<=0)or(IDL.Strings[id]=ID0); // check for ID match
  if(id<0)or(id=0)and(IDL.Strings[0]<>ID0)then beep // beep if no match found
    else IDL.Delete(id); // delete the self ID from the list of linked IDs
  RplInt(IDL.Count,2,OptiLine); // update the ID count in OptiLine
  OptiLine:= OptiLine+IDL.DelimitedText;
  OptiList[ci]:= OptiLine; // save changes to the processed line
end;
finally IDL.free; end;end; // release memory used by the local stringlists

// Now go calculate link lengths.
// SchField format by index: 0: H x V text, 1: header: 2+: data, V+3: header
// IDpinsNets: ID x y LinksLen LinksCount IDpinsNets
// ******** need to make a new list CalcLen *********
// CalcLen: ID x y Len #IDs IDsList (if x=0 then dx=0, if y=0 then dy=0)
// NetIDsList: nType nName nLen nLinks Symbols
procedure SymLinkLengths_HxV(var IDpinsNets,SchGrid: TStringList; H,V: integer);
  var Srt,Nxt,IDx: string; i,id,IDs,x,y,s,len: integer;
begin with IDpinsNets do for i:= 0 to Count-1 do begin
  Srt:= IDpinsNets[i]; len:= 0;
  x:= nt(CpyWrd(2,Srt)); // get the Xpos from IDpinsNets
  y:= nt(CpyWrd(3,Srt)); // get the Ypos from IDpinsNets
  IDs:= nt(CpyWrd(5,Srt)); // get the num of IDs from IDpinsNets
  for id:= 1 to IDs do begin
    IDx:= CpyWrd(5+id,Srt); // get the next linked ID from IDpinsNets
    s:=Count; repeat dec(s); Nxt:=IDpinsNets[s]; until(IDx=CpyWrd(1,Nxt))or(s<=0);
    if(s=0)and(IDx<>CpyWrd(1,Nxt))then beep else // add in dx+dy from IDpinsNets
      inc(len,abs(x-nt(CpyWrd(2,Nxt)))+abs(y-nt(CpyWrd(3,Nxt))));
  end;
//  RplInt(len,4,Srt); // update the linksLen in IDpinsNets
  IDpinsNets[i]:= Srt;
end;end;


