unit Builder;
{$mode objfpc}{$H+}
interface // this section makes stuff visible to other units
uses Classes, SysUtils, StdCtrls, StrUtils, SchBldrUtils;

procedure BuildSchematic
  (BlockBtn: boolean; var CfgMemo,NetMemo,BldMemo,SchMemo: TMemo);


implementation // anything appearing only here is invisible to other units
//uses SchBldrMain; // if needed must go here to prevent circular reference

var
  // these are the global aliases for the various TMemo arrays of Lines
  CfgList,NetList,BldList,SchList: TStrings;
  // these are the user configuration "knobs" set to default values
  TxtSp: integer=2; SymSp: integer=12; mCols: integer=5; mRows: integer=15;
  CfgError,NoSymbolFound: boolean; // flags for bad settings
  SymStr,NetStr,NetName,ID2,dotAsy,RefDes,Value,XY: string;
  Syms,Pins,Rtns, // symbol, pin and text-line-return counts
  Xsym,Ysym,Xpin,Ypin, // symbol and pin offsets
  xz,x,xm,dx,dxm, // these are the symbol x zero, position, increment and max
  yz,y,ym,dy,dym, // these are the symbol y zero, position, increment and max
  yt,dt, // these are the text y position and increment (xt is always zero)
  H,V, // auto derived dimensions for block schematic (MUST match Annealer)
  i,j,k: integer; // general purpose indices

procedure BuildSchematic
  (BlockBtn: boolean; var CfgMemo,NetMemo,BldMemo,SchMemo: TMemo);
begin // building the schematic
  // these create the aliases for the various TMemo arrays of Lines
  CfgList:= CfgMemo.Lines; NetList:= NetMemo.Lines;
  BldList:= BldMemo.Lines; SchList:= SchMemo.Lines;
  // Tstrings1:= Tstrings2 1 becomes alias for 2 (same Tstrings)
  // Tstrings1:= TstringList.Create sets up memory 1 points to
  // Tstrings1.assign(Tstrings2) clear1 then add2
  // Tstrings1.addstrings(Tstrings2) add2 to end of existing1
  Value:= CfgList.Strings[1]; // should contain user's output format data
  CfgError:= not(FoundInRange('Text',TxtSp,1,10,Value));
  CfgError:= not(FoundInRange('Symbols',SymSp,5,50,Value))or CfgError;
  CfgError:= not(FoundInRange('Symbols/Row',mCols,2,20,Value))or CfgError;
  CfgError:= not(FoundInRange('Rows',mRows,8,80,Value))or CfgError;
  xz:= 0; x:= 0; dx:= SymSp; yt:=0; dt:= TxtSp;
{*}if BlockBtn // was the block or inline button clicked?
    then begin Syms:= 0; // first count the valid symbols
      for i:= 1 to NetList.Count-1 do begin // line[0] is always a comment?
        NetStr:= Trim(NetList[i]); // copy, trim and check the non empty lines
        if(Length(NetStr)>0)then if(NetStr[1]in DeviceIDs) then inc(Syms); end;
      CalcHV(H,V,Syms); // find the schematic dimensions from the symbol count
      dym:= dy*V; yz:= -V*SymSp; y:= yz; dy:= dx; end // for Block
    else begin dxm:= dx*mCols; y:= 0; dy:= dx div 2; end; // for Inline
  xm:= dxm; ym:= dym;
  BldList.Clear; Syms:= 0; // Syms = symbol count
  SchList.Clear;
  SchList.Append('Version 4');       // schematic header
  SchList.Append('SHEET 1 960 720'); // schematic header
  for i:=1 to NetList.count-1 do begin // checking the netlist
    NetStr:= Trim(NetList.strings[i]); j:=3; // reset the symbol index
{*} if BlockBtn // Block only, check if it's an empty block position
      then if(length(NetStr)>0)and(NetStr[1]='~') // tilde is the empty flag
        then begin inc(y,dy); // top to bottom, left to right for Block
        if y>=ym then begin y:= yz; inc(x,dx); end; continue; end
      // Inline only, check schematic section column wrap for large netlists
      else if(x=xz)and((abs(y)+yt div 2)>(mRows*abs(dy)*2))
        then begin y:= 0; yt:= 0; inc(xz,dxm+dx); inc(xm,dxm+dx); x:= xz; end;
    k:= Pos('(',NetStr); if k>0 then Insert(' ',NetStr,k); // fix "poly()"
    // start building the schematic symbols here
    repeat inc(j); NoSymbolFound:= (j>=CfgMemo.Lines.Count);
      SymStr:= CfgList.strings[j];
      if (length(SymStr)=0)or not(SymStr[1]in DeviceIDs) // ['*',';','-']-not
        then continue; // the configuration line is empty or a comment
      if(length(NetStr)>0)and(SymStr[1]=UpCase(NetStr[1]))
      then begin   // found an ID match, so start checking 2nd ID
        ID2:= ExtractWord(2,SymStr,WhtSp);
        Pins:= nt(ExtractWord(6,SymStr,WhtSp));
        Value:= ExtractWord(2+Pins,NetStr,WhtSp);
        if (Value='')and(SymStr[1]in['M','Q']) then begin
          dec(Pins); Value:= ExtractWord(2+Pins,NetStr,WhtSp);
        end; // BJTs and MOSFETs may be only 3 pins (this needs work)
        if (ID2='.')or(Pos(ID2,LowerCase(Value))=1) then begin // (ID2 match)
          BldList.Append(NetStr); // tell Build page that a symbol is found
{*}       if BlockBtn // select proper symbol placement direction...
            then if y=yz then {inc(x,dx)}else
            else if x=xz then inc(y,dy);  // half space the new symbol line
          XY:= ExtractWord(5,SymStr,WhtSp);
          Xsym:= (x-nt(ExtractWord(1,XY,[','])))*grid;
{*}       if BlockBtn // here we account for the heigth of inline text
            then Ysym:= (y-nt(ExtractWord(2,XY,[','])))*grid           // Block
            else Ysym:= (y-nt(ExtractWord(2,XY,[','])))*grid+yt*tgrid; // Inline
          RefDes:= ExtractWord(1,NetStr,WhtSp);
          dotAsy:= ExtractWord(3,SymStr,WhtSp)+' ';
          if SymStr[1]='A' then Value:= ''; // already ID'd by SpiceModel
          for k:= 3+Pins to WordCount(NetStr,WhtSp) do
            Value:= Value+' '+ExtractWord(k,NetStr,WhtSp);
          SchList.Append('SYMBOL '+dotAsy+st(Xsym)+st(Ysym)+'R0');
          SchList.Append('SYMATTR InstName '+RefDes);
          SchList.Append('SYMATTR Value '+Value); // got this back with pins
          for k:= 1 to Pins do begin // write netlabels on pins
            XY:= ExtractWord(6+k,SymStr,WhtSp);
            if XY<>',' then begin
              Xpin:= nt(ExtractWord(1,XY,[',']))*grid;
              Ypin:= nt(ExtractWord(2,XY,[',']))*grid;
              NetName:= ExtractWord(1+k,NetStr,WhtSp);
              if not((SymStr[1]='A')and(NetName='0')) then // no A-device GND
                SchList.Append('FLAG '+st(Xsym+Xpin)+st(Ysym+Ypin)+NetName);
          end;end;
{*}       if BlockBtn // select proper symbol placement direction...
            then begin inc(y,dy); // top to bottom, left to right for Block
              if y>=ym then begin y:= yz; inc(x,dx); end;end
            else begin inc(x,dx); // left to right, top to bottom for Inline
              if x>=xm then begin x:= xz; inc(y,dy); end;end; // half space
          inc(Syms); break; // found a symbol, so quit looking further
      end;end;
    until NoSymbolFound; // symbol not found so process as text...
    if (length(NetStr)>0)and(NoSymbolFound)then begin // ...unless empty
      BldList.Append(NetStr); // progress message to Build page
      inc(yt,dt); // half space the new text line
      if NetStr[1]='*' then begin  // these are comments
        Delete(NetStr,1,1); Trim(NetStr);
        NetStr:= 'Left 2 ;'+NetStr; end else
        NetStr:= 'Left 2 !'+NetStr; // these are directives
      Rtns:= 0; while NPos('\n',NetStr,Rtns+1)>0 do inc(Rtns);
{*}   with SchList do if BlockBtn // was the block or inline button clicked?
        then begin Append('TEXT '+st(xz*grid)+st(yt*tgrid)+NetStr); end
        else begin if x>xz then inc(y,dy); x:= xz;
          Append('TEXT '+st(xz*grid)+st(y*grid+yt*tgrid)+NetStr); end;
      inc(yt,dt+Rtns*4); // half space + full space returns
    end;
  end;// of netlist check loop
  with BldList do begin
    Append(' - - -');
    Append('Total symbols rendered: '+st(Syms));
  end;
  if CfgError then with BldList do begin beep;
    Append('');
    Append('WARNING! Problem with second line of configuration page!');
    Append('Some schematic format data is not recognizable or is out of range.');
    Append('Values have been limited or default values have been used as needed.');
  end;
end;

end. // anything beyond this point is ignored (notes and archive area)

BlkCol: integer=10;
CfgError:= not(FoundInRange('Block',BlkCol,5,50,Value))or CfgError;

// Schematic Builder pseudo-code (follows in last of Form Controls section)
//
// Upon Form creation ".create" TstringLists used in building schematic.
// To start new schematic, ".clear" schematic string list (schTSL).
// Append on new schematic standard header lines (everything is ASCII):
//   Version 4
//   SHEET 1 <w> <h> ; w,h are multiples of a 16 point grid (e.g. “960 720”)
// Work through each line of netlist, cross-checking the first letter to the
// list of recognized symbols in the configuration list.  When a match is found
// if a secondary match is necessary, find the substring and check for match.
// If a symbol is recognized then append its lines to schematic string list:
//   SYMBOL <name> <x> <y> <rot> ; e.g., SYMBOL res 96 80 R0 - (R0 R90, etc.)
//   SYMATTR InstName <RefDes> ; e.g., SYMATTR InstName R1
//   SYMATTR Value <Value> ; e.g., SYMATTR Value 1k
// Then parse pin offsets <dx> <dy> in symbol file in LTspice sym folder.
// Parse corresponding net names from the current netlist line and generate
// name flags at each pin:
//   FLAG <x+dx> <y+dy> <net> ; e.g., FLAG 160 80 Vcc
// Increment x,y location on schematic by increment in configuration file:
//   sx:= sx+inc,  if sx>swidth then sx:=0 and sy:=sy+inc;
// Repeat process for the next symbol until no more symbols are found.
// When done processing symbols, redo line-by-line netlist search for text and
// comments and append to the end of the schematic string list:
//   TEXT <x> <y> <orientation> <size> <Value> ; e.g., TEXT 112 248 Left 2 !.op
//
// When done with text end process (for now), but consider these refinements:
// * Generate sorted list of net names by connection count.
// * Rotate components so Gnd pins are down and Vcc pins are up.
// * Change all net names with count greater than ten percent to Flags.
// * Use "synthetic annealing" to reposition components for shortest total
//   of length remaining "air" wires.
// * Fill in "air" wires adding bends as necessary to avoid misconnections.

