unit Optimizer;
{$mode objfpc}{$H+}
interface // this section makes stuff visible to other units
uses Classes, SysUtils, StdCtrls, StrUtils, SchBldrUtils;

procedure OptimizeSchematic(var CfgMemo,NetMemo,BldMemo,AnlMemo,SchMemo: TMemo);


implementation // anything appearing only here is invisible to other units
//uses SchBldrMain; // if needed must go here to prevent circular reference

///////////////////////////////////////////////////////////////////////////////
//
//  SrtTSL: ID Xpos Ypos Len Pins Connection_Names (all text fields)
//  StatTSL: NType NetID Len Links Symbols (net connectivity statistics)
//           NType, Net Name, Length, Links*, Symbols (sorted by link count)
//  SchTSL: the display (V+1 tab filled lines of H+1 positions of symbols)
//    note: this could just be part of the memo lines
//
// When optimizing, lock out changes to memo, especially adding lines, so that
// the schematic field offset does not change (or have the offset track the
// change).  Do not maintain a separate list for the schematic field.
//
// When optimizing, swap locations in the field (including empty points) and
// keep if link length decreases.

type // symbols is the total number of actual symbols, ID=0 is a null filler
  IDs = integer; names = string; point = record x,y: integer end;
  ODs = array of integer; // [1..symbols-1]
  Nets = (Port, Top, Bot, Net, NetL, NetR);
  SwapData = record NType: nets; xfree, yfree: boolean; end;
const
  symbols=10; H=10; V=10;
var // the empty positions in the field array all get the null ID (=0)
  SF: array of ODs; // key to finding symbols to be swapped [0..H,0..V]
  SD: array of SwapData; // key to symbol swap restrictions [1..symbols]
  XY: array of point; // key to computing link lengths [1..symbols]
  OD: array of ODs; // key computing link lengths [1..symbols]
{
function LnkLen(ID: integer): integer; var i: integer;
  begin LnkLen:= 0;
    if ID>0 then for i:=1 to count(ODs) do
      inc(LnkLen,abs(XY[ID].x-XY[OD[i]].x)+abs(XY[ID].y-XY[OD[i]].y));
  end;

procedure SetContraints; var i: integer;
  begin // Ntypes: Port,Top,Bot,Net,NetL,NetR
    with SD[0] do begin xfree:= true; yfree:= true; end;
    for i:=1 to symbols do with SD[i] do case Ntype of
      Port: begin xfree:= true;  yfree:= true;  end;
      Top : begin xfree:= true;  yfree:= false; end;
      Bot : begin xfree:= true;  yfree:= false; end;
      Net : begin xfree:= true;  yfree:= true;  end;
      NetL: begin xfree:= false; yfree:= true;  end;
      NetR: begin xfree:= false; yfree:= true;  end;
    end;
  end;

procedure Swap(P1,P2: point); var P3: point; ID1,ID2: integer;
  begin
    ID1:= SF[P1.x,P1.y]; ID2:= SF[P2.x,P2.y];
    if((SD[ID1].xfree and SD[ID2].xfree)or(P1.x=P2.x))and
      ((SD[ID1].yfree and SD[ID2].yfree)or(P1.y=P2.y))
      then begin P3:= P2; P2:= P1; P1:= P3; end;
  end;
// var TotalLnkLen: integer;
// TotalLnkLen:= 0; for i:= 1 to symbols do inc(TotalLnkLen, LnkLen(i));

// draw the entire schematic symbol placement optomization field
// SchTSL format by index: 0: H x V text, 1: header: 2+: data, V+3: header
procedure DrawIDSinSchHV(var IDsTSL,SchTSL: TStringList; H,V: integer);
var Hdr,Sch: string; i,j,k: integer;
begin // first create correct length header/footer
  Hdr:=' +'#9; for i:=0 to H do Hdr:= Hdr+'- -'#9; Hdr:= Hdr+'+';
with SchTSL do begin k:= 0;
  Append('Schematic Symbol Field: '+st(H+1)+'x '+st(V+1));
  Append(Hdr); // write top boarder of field
  for i:= 0 to V do begin Sch:='  :'#9; // que left boarder of field
    for j:= 0 to H do begin
      if (k)>=Symbols then SymStr:= '' else begin
        SrtStr:= SrtTSL[k];
        SymStr:= ExtWrd(1,SrtStr); // get the IDs in order from SrtTSL
        InsInt(j+1,2,SrtStr); // update the X grid position in SrtTSL
        InsInt(i+1,3,SrtStr); // update the Y grid position in SrtTSL
        // calculate branch length here
        SrtTSL[k]:= SrtStr;
      end; inc(k); // move to next symbol
      end;
      Sch:= Sch+SymStr+#9; // add symbols to left boarder of field
    end; Sch:= Sch+':'; // add in right boarder of field
    Append(Sch); // write middle lines of field
  Append(Hdr); end; // write bottom boarder of field
end;

// stuff the RefDes (or blank, if =0) into the schematic display matrix at XY
// SchTSL format by index: 0: H x V text, 1: header: 2+: data, V+3: header
procedure StuffIDinSchXY(Item: ID; var SchTSL: TStringList; P: point);
var len,pos: integer; txt: string;
begin
  wrd:= P.x; st:= SchTSL[P.y+1]; // translate x,y grid position
  len:= length(ExtractWordPos(wrd,st,WhtSp,pos)); // old position & length
  Str(item,txt); // convert the new item(ID) to txt (item is integer?)
  st:= StuffString(st,pos,len,txt); // replace the old with the new
  SchTSL[P.y+1]:= st; // put the updated line back into the string list
end;
}
procedure OptimizeSchematic(var CfgMemo,NetMemo,BldMemo,AnlMemo,SchMemo: TMemo);
var n,ID1,ID2,OldLnkLen,NewLnkLen: integer; P1,P2: point;
const count=500;
begin n:= 0;
{  repeat inc(n); // note: ID=0 means "no symbol here"
    with P1 do begin x:= random(H+1); y:= random(V+1) end;
    with P2 do begin x:= random(H+1); y:= random(V+1) end;
    ID1:= SF[P1.x,P1.y]; ID2:= SF[P2.x,P2.y];
    OldLnkLen:= LnkLen(ID1)+LnkLen(ID2); Swap(P1,P2);
    NewLnkLen:= LnkLen(ID1)+LnkLen(ID2);
    if NewLnkLen<OldLnkLen
      then begin n:= 0; StuffIDinSchXY(ID1,SchTSL,P1); end
      else if NewLnkLen<OldLnkLen then Swap(P1,P2);
  until n>=count;
}
end;

end.

(**************************** Scratch Pad Area *******************************)

// Sch format by index: 0: H x V text, 1: header: 2+: data, V+3: header
// SrtTSL: ID x y LinksLen LinksCount LinksList
// ******** need to make a new list CalcLen *********
// CalcLen: ID x y Len #IDs IDsList (if x=0 then dx=0, if y=0 then dy=0)
// NetStatsList: nType nName nLen nLinks Symbols
procedure OptimizeSchematic(var CfgMemo,NetMemo,BldMemo,AnlMemo,SchMemo: TMemo);
var n,ID1,ID2,OldLnkLen,NewLnkLen: integer; P1,P2: point;
const itterations=500;
begin n:= 0;
  repeat inc(n); // note: ID=0 means "no symbol here"
    X1:= random(H+1); Y1:= random(V+1);
    ID1:= FindID(X1,Y1,TSL); NTyp1:= UpCase(ExtWrd(1,NSL[ID1]));
    Xfixed:= (NTyp1='RHT')or(NTyp1='LFT');
    Yfixed:= (NTyp1='TOP')or(NTyp1='BOT');
    if Yfixed then Y2:= Y1 else Y2:= random(V+1);
    if Xfiexd then X2:= X1 else X2:= random(H+1);
    ID2:= FindID(X2,Y2,TSL); NTyp2:= UpCase(ExtWrd(1,NSL[ID2]));
    if(Xfixed)and((NTyp2='TOP')or(NTyp2='BOT'))then continue;
    if(Yfixed)and((NTyp2='RHT')or(NTyp2='LFT'))then continue;
    L1:= LenXY(X2,Y2,ID1,TSL); L2:= LenXY(X1,Y1,ID2,TSL);
    if(L1+L2)<(ExtWrd(4,TSL[ID1])+ExtWrd(4,TSL[ID1]))
      then begin // swap data
        Exchange(ID1,ID2,TSL); // swap position?
        InsInt(X1,2,TSL[ID1]); InsInt(X2,2,TSL[ID2]); // stuff new X
        InsInt(Y1,3,TSL[ID1]); InsInt(Y2,3,TSL[ID2]); // stuff new Y
        InsInt(L1,4,TSL[ID1]); InsInt(L2,4,TSL[ID2]); // stuff new Len
        n:= 0; // reset counter of itterations without improvement
       end;
  until n>=itterations;
end;

// NType Key 	Connectivity	Optimization Weighting
// ------------	---------------	---------------------------
// Port:        port flags   	not used (zero length)
// Top:		top edge   	vertical length only
// Bot:		bottom edge 	vertical length only
// Net:		node to node 	vertical and horizontal length
// NetL:	left end node 	pinned to left end
// NetR:	right end node 	pinned to right end

