unit SchBldrMain;
{$mode objfpc}{$H+}
interface // this section makes stuff visible to other units
uses
  Classes, SysUtils, FileUtil, Forms, Controls, Graphics, Dialogs,
  ComCtrls, StdCtrls, Menus, ActnList, ExtCtrls, Buttons,
  Builder, Annealer;

type { TForm1 }
  TForm1 = class(TForm)
    SelectDirectoryDialog1: TSelectDirectoryDialog;
    OpenDialog1: TOpenDialog; SaveDialog1: TSaveDialog;
    PageControl1: TPageControl;
    CfgSht,NetSht,BldSht,AnlSht,SchSht,HlpSht: TTabSheet;
    CfgMemo,NetMemo,BldMemo,AnlMemo,SchMemo,HlpMemo: TMemo;
    CfgLoadBtn,CfgSaveBtn,CfgBrowseBtn,NetBtn,BlockBtn,InlineBtn,
      AnalyzeBtn,OptimizeBtn,SchBtn: TButton;
    CfgSaveLbl,CfgBrowseLbl,NetLbl,SchLbl,HlpLbl,RevLbl,
      BlockLbl,InlineLbl,AnalyzeLbl,OptimizeLbl: TLabel;
    function SymDirExists: boolean;
    procedure FormCreate(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure FormClose(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure PageControl1Changing(Sender: TObject; var AllowChange: Boolean);
    procedure CfgLoadBtnClick(Sender: TObject);
    procedure CfgSaveBtnClick(Sender: TObject);
    procedure CfgBrowseBtnClick(Sender: TObject);
    procedure CfgMemoChange(Sender: TObject);
    procedure NetBtnClick(Sender: TObject);
    procedure NetMemoChange(Sender: TObject);
    procedure BldBtnClick(Sender: TObject);
    procedure AnnealBtnClick(Sender: TObject);
    procedure SchBtnClick(Sender: TObject);
    procedure SchMemoChange(Sender: TObject);
  private { private declarations }
  public { public declarations }
  end; { TForm1 type definition }

var Form1: TForm1; App1: TApplication;
  SymDir: string; // global for all units
const CfgFile = 'SchBuilder.cfg';

implementation // anything appearing only here is invisible to other units
// TForm1 - all TForm1 controls are located within this section
{$R *.lfm}

function TForm1.SymDirExists: boolean; // maybe move this elsewhere?
  var n,f,x: integer; str: string;
begin
  str:= CfgMemo.Lines.Strings[0]; n:= Pos('"',str)+1;
  SymDir:= copy(str,n,length(str)-n);
  f:= Pos(' Files',SymDir); x:= Pos(' (x86)',SymDir);
  if (not DirectoryExists(SymDir))and(f>0) then if (x>0)
    then Delete(SymDir,x,6) else Insert(' (x86)',SymDir,f+6);
  if DirectoryExists(SymDir) then begin SymDirExists:= true;
    CfgBrowseBtn.Visible:= false; CfgBrowseLbl.Visible:= false;
    CfgMemo.BorderSpacing.Top:= CfgLoadBtn.Height;
    CfgMemo.Lines.Delete(0);
    CfgMemo.Lines.Insert(0,'Symbol folder: "'+SymDir+'"');
    CfgMemo.Font.Color:= clDefault;
  end else begin
    SymDirExists:= false;
    CfgBrowseBtn.Visible:= true; CfgBrowseLbl.Visible:= true;
    CfgMemo.BorderSpacing.Top:= CfgLoadBtn.Height+CfgBrowseBtn.Height;
  end;
end;

procedure TForm1.FormCreate(Sender: TObject);
begin // do initialization stuff here
  Randomize; // seed the random number generator
  HlpMemo.WordWrap:= true; // must initially be false in Object Inspector
  SaveAnnealPageSetUp(AnlMemo); // save the unchanging lines
  CfgLoadBtnClick(Form1); // load configuration if available
end;

procedure TForm1.FormActivate(Sender: TObject);
begin
  if SymDirExists
    then PageControl1.TabIndex:= 5 // open About tab at start up
    else begin PageControl1.TabIndex:= 0; beep; end;
end;

procedure TForm1.FormShow(Sender: TObject);
begin
  CfgMemo.BorderSpacing.Top:= CfgLoadBtn.Height;//+CfgBrowseBtn.Height;
  NetMemo.BorderSpacing.Top:= NetBtn.Height;
  BldMemo.BorderSpacing.Top:= BlockBtn.Height+InlineBtn.Height;
  SchMemo.BorderSpacing.Top:= SchBtn.Height;
  AnlMemo.BorderSpacing.Top:= AnalyzeBtn.Height+OptimizeBtn.Height;
end;

procedure TForm1.FormClose(Sender: TObject);
begin
  if (CfgMemo.Font.Color<>clDefault) then with SaveDialog1 do begin beep;
    Title:= 'Configuration data has changed! Save file as?';
    InitialDir:= FindDefaultExecutablePath('SchBuilder');
    Filter:= 'Configuration File |*.cfg';
    FileName:= 'SchBuilder.cfg'; Execute;
    Options:= [ofEnableSizing,ofViewDetail,ofOverwritePrompt];
    if FilenameIsAbsolute(FileName) then CfgMemo.Lines.SaveToFile(FileName);
  end;
end;

procedure TForm1.PageControl1Changing(Sender: TObject; var AllowChange: Boolean);
begin
  if (CfgMemo.Font.Color<>clDefault) and not SymDirExists
    then begin AllowChange:= false; beep; end
    else AllowChange:= true;
end;

procedure TForm1.CfgMemoChange(Sender: TObject);
begin
  CfgMemo.Font.Color:= clHighlight;
end;

procedure TForm1.CfgLoadBtnClick(Sender: TObject);
var fpath: string;
begin
  fpath:= ProgramDirectory;
  if FileExists(fpath+CfgFile) then begin;
    CfgMemo.Lines.LoadFromFile(fpath+CfgFile);
    CfgSaveLbl.Caption:= 'Loaded: '+fpath+CfgFile;
    SymDirExists;
  end else begin
    CfgSaveLbl.Caption:= '(config file not found, using default)';
  end;
  CfgMemo.Font.Color:= clDefault;
end;

procedure TForm1.CfgSaveBtnClick(Sender: TObject);
var fpath: string;
begin
  fpath:= ProgramDirectory;
  if DirectoryExists(fpath) then begin
    SymDirExists;
    CfgMemo.Lines.SaveToFile(fpath+CfgFile);
    CfgSaveLbl.Caption:= 'Saved: '+fpath+CfgFile;
    CfgMemo.Font.Color:= clDefault;
  end;
end;

procedure TForm1.CfgBrowseBtnClick(Sender: TObject);
begin with SelectDirectoryDialog1 do begin
  Title:= 'Select LTspice symbol folder';
  Execute;
  SymDir:= FileName;
  if DirectoryExists(SymDir) then with CfgMemo.Lines do begin
    Delete(0); Insert(0,'Symbol folder: "'+SymDir+'"'); end;
  SymDirExists;
end; end;

procedure TForm1.NetBtnClick(Sender: TObject);
begin with OpenDialog1 do begin
  Title:= 'Open existing SPICE Netlist file';
  Filter:= 'Netlists |*.cir; *.net; *.sp|All Files |*.*';
  Options:= [ofEnableSizing,ofViewDetail,ofFileMustExist];
  Execute;
  if FileExists(FileName) then begin
    NetMemo.Lines.LoadFromFile(FileName);
    NetLbl.Caption:= 'Opened: '+FileName;
    NetMemo.Font.Color:= clDefault;
  end else
    NetLbl.Caption:= 'ERROR: File not opened!';
end end;

procedure TForm1.NetMemoChange(Sender: TObject);
begin
  NetMemo.Font.Color:= clHighlight;
end;

procedure TForm1.BldBtnClick(Sender: TObject);
begin // build the schematic into the Schematic tab Memo
  BuildSchematic(Sender=BlockBtn,CfgMemo,NetMemo,BldMemo,SchMemo);
  // send over which button was clicked
  SchMemo.Font.Color:= clDefault; // No user edits yet, so any changes
  NetMemo.Font.Color:= clDefault; // <- here are assumed to be wanted.
end;

procedure TForm1.AnnealBtnClick(Sender: TObject);
begin
  if(Sender=AnalyzeBtn)
    then
      if(OptimizeBtn.Enabled=false)
        then begin
          OptimizeBtn.Enabled:= true;
          OptimizeLbl.Enabled:= true;
          AnalyzeSchematic(CfgMemo,NetMemo,AnlMemo);//AnalyzeOptimize(true,false);
        end else AnalyzeSchematic(CfgMemo,NetMemo,AnlMemo)//AnalyzeOptimize(true,true)
    else OptimizeSchematic(AnlMemo);//AnalyzeOptimize(false,true);
end;

procedure TForm1.SchBtnClick(Sender: TObject);
begin with SaveDialog1 do begin
  //InitialDir:= FindDefaultFilePath(NetFile);
  Title:= 'Save LTspice schematic file as';
  Filter:= 'LTspice schematic file |*.asc';
  FileName:= 'SchBuilder.asc';
  Options:= [ofEnableSizing,ofViewDetail,ofOverwritePrompt];
  if Execute then begin
    SchMemo.Lines.SaveToFile(FileName);
    SchLbl.Caption:= FileName; //ExpandFileName(FileName);
    SchMemo.Font.Color:= clDefault;
    NetMemo.Font.Color:= clDefault;
  end;
end; end;

procedure TForm1.SchMemoChange(Sender: TObject);
begin
  SchMemo.Font.Color:= clHighlight;
end;

end. // anything beyond this point is ignored (notes and archive area)

// Alternate Anneal Tab Labels? Anneal Align Tune Sort Tidy Clean Reduce

